//
//  ExtraItemsListViewController.m
//  scummvm
//
//  Created by ilya on 12/18/15.
//
//

#import "ExtraItemsListViewController.h"
#import "UIStoryboard+scummVMAdditions.h"
#import "NSStringAdditions.h"
#import "UIActivityViewController+MTAdditions.h"
#import <MediaPlayer/MediaPlayer.h>
#import "Strings.h"
#import "NSBundle+MTAdditions.h"

using namespace Enhanced;

@interface ExtraItemsListViewController ()
@property (nonatomic) vector<ExtraItem> extraItems;
@property (nonatomic, strong) UIDocumentInteractionController* interactionController;

@end

@implementation ExtraItemsListViewController

+(instancetype)extrasListWithItems:(vector<ExtraItem>)items
{
    ExtraItemsListViewController* extraViewController = [[UIStoryboard mainStoryboard] instantiateViewControllerWithIdentifier:@"ExtraItemsListViewController"];
    
    NSMutableArray* itemStrings = [[NSMutableArray alloc] init];
    
    std::for_each(items.begin(), items.end(), ^(ExtraItem item){
        [itemStrings addObject:[NSString stringWithCPPWString:item.mTitle]];
    });
    
    extraViewController.items = itemStrings;
    extraViewController.extraItems = items;
    
    return extraViewController;
}


- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.
    
    self.tableView.alwaysBounceVertical = NO; //disable scrolling when not needed
    

    
}

-(void)configureButtonTitles
{
    NSString* backTitle = [NSString stringWithCPPWString:Strings::getString(StringId::back)];
    
    self.negativeButton = backTitle;
    
    [super configureButtonTitles];
}

-(void)indexPathTapped:(NSIndexPath *)indexPath
{
    ExtraItem item = self.extraItems[indexPath.row];
    
    switch (item.mType)
    {
        case VIDEO: {
            // play
            [self playVideoItem:item];
            break;
        }
        case ExtraItemType::MUSIC: {
            // play
            [self playMusicItem:item];
            break;
        }
        case LOCAL_FILE: {
            // try to open
            [self openLocalFile:item];
            break;
        }
        case LINK: {
            // open
            [self openLinkItem:item];
            break;
        }
        case LIST: {
            // show sublist
            [self showSublist:item];
            break;
        }
    }
    
    [self.tableView deselectRowAtIndexPath:indexPath animated:YES];
}

-(void)playVideoItem:(ExtraItem)item
{
    NSURL* url = [[NSBundle currentGameBundle] URLForResource:[NSString stringWithCPPString:item.mContent] withExtension:nil];
    
    MPMoviePlayerViewController* moviePlayerVC = [[MPMoviePlayerViewController alloc] initWithContentURL:url];
    
    moviePlayerVC.moviePlayer.controlStyle = MPMovieControlStyleNone;
    
    [self presentViewController:moviePlayerVC animated:YES completion:^{
        [[moviePlayerVC moviePlayer] play];
    }];
}

-(void)playMusicItem:(ExtraItem)item
{
    
}

-(void)openLinkItem:(ExtraItem)item
{
    NSURL* url = [NSURL URLWithString:[NSString stringWithCPPString:item.mContent]];
    
    [[UIApplication sharedApplication] openURL:url];
}

-(void)openLocalFile:(ExtraItem)item
{
    NSURL* url = [[NSBundle currentGameBundle] URLForResource:[NSString stringWithCPPString:item.mContent] withExtension:nil];
    
    UIDocumentInteractionController* documentInteractionController = [UIDocumentInteractionController interactionControllerWithURL:url];
    
    [documentInteractionController presentOpenInMenuFromRect:self.tableView.frame inView:self.view animated:YES];
    self.interactionController = documentInteractionController;
}

-(void)showSublist:(ExtraItem)item
{
    ExtraItemsListViewController* extralist = [ExtraItemsListViewController extrasListWithItems:item.mSubItems];
    
    [self.navigationController pushViewController:extralist animated:YES];
}

-(IBAction)share:(id)sender
{
    UIActivityViewController* activityViewController = [UIActivityViewController shareMyAppActivityViewController];
    BOOL isIpad = [UIDevice currentDevice].userInterfaceIdiom == UIUserInterfaceIdiomPad;
    
    if (isIpad) {
        UIPopoverController* popoverController = [[UIPopoverController alloc] initWithContentViewController:activityViewController];
        
        [popoverController presentPopoverFromRect:[sender frame] inView:self.view permittedArrowDirections:UIPopoverArrowDirectionAny animated:YES];
    }
    else
    {
        [self presentViewController:activityViewController animated:YES completion:nil];
    }
}

-(void)negativeTapped:(id)sender
{
    [self.navigationController popViewControllerAnimated:YES];
}


@end
