//
//  InputDialogViewController.m
//  scummvm
//
//  Created by Ilya Levin on 17/10/2015.
//
//

#import "InputDialogViewController.h"
#import "UIStoryboard+scummVMAdditions.h"
#import "NSStringAdditions.h"
#import "UIButton+MTAdditions.h"
#import "UILabel+MTAdditions.h"
#import "UIColor+MTAdditions.h"
#import "UIFont+ScummVMAdditions.h"
#include "Appirater.h"
// Just for logs
#include "common/textconsole.h"

@interface InputDialogViewController () <UITextFieldDelegate>
@property (retain, nonatomic) IBOutlet UILabel *lblMessage;
@property (retain, nonatomic) IBOutlet UITextField *textField;
@property (retain, nonatomic) IBOutlet UIButton *btnNegative;
@property (retain, nonatomic) IBOutlet UIButton *btnPositive;

@property (nonatomic, strong) NSString* message;
@property (nonatomic, strong) NSString* initialInput;
@property (nonatomic, strong) NSString* negativeButton;
@property (nonatomic, strong) NSString* positiveButton;
@property (nonatomic) shared_ptr<Enhanced::InputDialogListener> listener;

@end

@implementation InputDialogViewController

+(instancetype)inputeDialogViewContollerWithMessage:(NSString *)message intialInput:(NSString *)initialInput negativeButton:(NSString *)negativeButton positiveButton:(NSString *)positiveButton listener:(shared_ptr<Enhanced::InputDialogListener>)listener
{
    InputDialogViewController* inputViewController = [[UIStoryboard mainStoryboard] instantiateViewControllerWithIdentifier:@"InputDialogViewController"];
    
    inputViewController.message = message;
    inputViewController.initialInput = initialInput;
    inputViewController.negativeButton = negativeButton;
    inputViewController.positiveButton = positiveButton;

    inputViewController.listener = listener;
    inputViewController.modalPresentationStyle = UIModalPresentationOverCurrentContext;
    
    return inputViewController;
}

#define MINLENGTH 1
#define MAXLENGTH 12

-(BOOL)textField:(UITextField *)textField shouldChangeCharactersInRange:(NSRange)range replacementString:(NSString *)string
{ // check the input is from the allowed characters and not larger than MAXLENGTH
    
  NSCharacterSet *numbersOnly = [NSCharacterSet characterSetWithCharactersInString:@"ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789 "];
    NSCharacterSet *characterSetFromTextField = [NSCharacterSet characterSetWithCharactersInString:string];
    
    NSUInteger oldLength = [textField.text length];
    NSUInteger replacementLength = [string length];
    NSUInteger rangeLength = range.length;
  //  NSLog("@Old=%s @Replacement=%s @Range=%s", oldLength, replacementLength, rangeLength);
    NSUInteger newLength = oldLength - rangeLength + replacementLength;
    
    BOOL stringIsValid = [numbersOnly isSupersetOfSet:characterSetFromTextField];
    if(stringIsValid){
        BOOL returnKey = [string rangeOfString: @"\n"].location != NSNotFound;
        if (newLength <= MAXLENGTH || returnKey){
            if (self.nativeUIInterface->hasFontwithLowercase()){
                return YES;
            }
            else{ // Forcing capital letters if needed
                // not so easy to get an UITextRange from an NSRange...
                // thanks to Nicolas Bachschmidt (see http://stackoverflow.com/questions/9126709/create-uitextrange-from-nsrange)
                UITextPosition *beginning = textField.beginningOfDocument;
                UITextPosition *start = [textField positionFromPosition:beginning offset:range.location];
                UITextPosition *end = [textField positionFromPosition:start offset:range.length];
                UITextRange *textRange = [textField textRangeFromPosition:start toPosition:end];
                
                // replace the text in the range with the upper case version of the replacement string
                [textField replaceRange:textRange withText:[string uppercaseString]];
                return NO;
            }
        }
        else{
            return NO;
        }
    }
    else{
        return NO;
    }
}

-(void)viewDidLoad
{
    [super viewDidLoad];
    
    self.view.backgroundColor = [[UIColor blackColor] colorWithAlphaComponent:0.6]; // Greyed out background
    
    if (self.nativeUIInterface->hasFontwithLowercase()){
        self.lblMessage.text = self.message;
        self.textField.text = self.initialInput;
        [self.btnPositive setTitle:self.positiveButton forState:UIControlStateNormal];
        [self.btnNegative setTitle:self.negativeButton forState:UIControlStateNormal];
    }
    else{ // Forcing capital letters if needed
        self.textField.autocapitalizationType = UITextAutocapitalizationTypeAllCharacters;
        [_lblMessage setText:[self.message uppercaseString]];
        [_textField setText:[self.initialInput uppercaseString]];
        [self.btnPositive setTitle:[self.positiveButton uppercaseString] forState:UIControlStateNormal];
        [self.btnNegative setTitle:[self.negativeButton uppercaseString] forState:UIControlStateNormal];
    }
    
    [self.btnNegative configureWithTheme:YES];
    [self.btnPositive configureWithTheme:YES];
    [self.lblMessage configureWithTheme];
    
    [self.textField setBackgroundColor:[UIColor clearColor]]; // Transparent background
    
    // Create a border for the textfield
    self.textField.layer.cornerRadius=8.0f;
    self.textField.layer.masksToBounds=YES;
    [[self.textField layer] setBorderColor:[[UIColor colorWithRed:171.0/255.0 green:171.0/255.0 blue:171.0/255.0 alpha:1.0] CGColor]];
    self.textField.layer.borderWidth= 2.0f;
    
    [self.textField becomeFirstResponder];
    self.textField.keyboardType = UIKeyboardTypeASCIICapable; // Only acceptable keyboards
    self.textField.font = [UIFont mt_themeFontOfAppropriateSize];
    self.textField.textColor = [UIColor mt_mainButtonColor];
    
}

- (IBAction)negativeTapped:(id)sender
{
    [buttonClickPlayer() play];

    self.listener->onResult(false, string());
    [self dismissViewControllerAnimated:NO completion:nil];

}

- (IBAction)positiveTapped:(id)sender
{
    [buttonClickPlayer() play];

    if (self.textField.text.length < MINLENGTH){ // If the input is empty - auto set the name of the save slot to be displayed: "NO NAME".
        self.textField.text=@"NO NAME";
    }
    
    self.listener->onResult(true, [self.textField.text cppString]);
    [self dismissViewControllerAnimated:NO completion:nil];
    [Appirater userDidSignificantEvent:YES];
}

@end
