//
//  MenuViewController.m
//  scummvm
//
//  Created by Ilya Levin on 10/10/2015.
//
//

#import "MenuViewController.h"
#import "SettingsViewController.h"
#import "UIFont+ScummVMAdditions.h"
#import "Strings.h"
#import "NSStringAdditions.h"
#import "UIColor+MTAdditions.h"
#import "UIButton+MTAdditions.h"
#import "AboutViewController.h"
#import "NSArray+CPPConvertionAdditions.h"
#import "ExtraItemsListViewController.h"
#import "UIActivityViewController+MTAdditions.h"

using namespace Enhanced;

@interface MenuViewController ()
@property (retain, nonatomic) IBOutlet UIButton *btnNewGame;
@property (retain, nonatomic) IBOutlet UIButton *btnSave;
@property (retain, nonatomic) IBOutlet UIButton *btnLoad;
@property (retain, nonatomic) IBOutlet UIButton *btnContinue;
@property (retain, nonatomic) IBOutlet UIButton *btnSettings;
@property (retain, nonatomic) IBOutlet UIButton *btnTutorial;
@property (retain, nonatomic) IBOutlet UIButton *btnAd;
@property (retain, nonatomic) IBOutlet UIButton *btnExtras;
@property (retain, nonatomic) IBOutlet UIButton *btnQuit;

@end

@implementation MenuViewController

-(void)viewDidLoad
{
    [super viewDidLoad];
    
    [self.navigationController setNavigationBarHidden:YES];

    [self configureButtons];
    
    [[UITableViewCell appearance] setBackgroundColor:[UIColor clearColor]];
    
    // load About webview in advance
    AboutViewController *aboutViewController;
    [aboutViewController.webView setNeedsLayout];
    [aboutViewController.webView layoutIfNeeded];
}

-(void)configureButtons
{
    NSString* newGameTitle;
    NSString* continueTitle;
    NSString* loadTitle;
    NSString* saveTitle;
    NSString* settingsTitle;
    NSString* quitTitle;
    NSString* extrasTitle;
    NSString* tutorialTitle;
    if (self.nativeUIInterface->hasFontwithLowercase()){
        newGameTitle = [NSString stringWithCPPWString:Strings::getString(StringId::new_game)];
        continueTitle = [NSString stringWithCPPWString:Strings::getString(StringId::continue_)];
        loadTitle = [NSString stringWithCPPWString:Strings::getString(StringId::load_game)];
        saveTitle = [NSString stringWithCPPWString:Strings::getString(StringId::save_game)];
        settingsTitle = [NSString stringWithCPPWString:Strings::getString(StringId::settings)];
        quitTitle = [NSString stringWithCPPWString:Strings::getString(StringId::quit)];
        extrasTitle = [NSString stringWithCPPWString:Strings::getString(StringId::extras)];
        tutorialTitle = [NSString stringWithCPPWString:Strings::getString(StringId::tutorial)];
    }
    else { // Forcing capital letters if needed

        newGameTitle = [[NSString stringWithCPPWString:Strings::getString(StringId::new_game)] uppercaseString];
        continueTitle = [[NSString stringWithCPPWString:Strings::getString(StringId::continue_)] uppercaseString];
        loadTitle = [[NSString stringWithCPPWString:Strings::getString(StringId::load_game)] uppercaseString];
        saveTitle = [[NSString stringWithCPPWString:Strings::getString(StringId::save_game)] uppercaseString];
        settingsTitle = [[NSString stringWithCPPWString:Strings::getString(StringId::settings)] uppercaseString];
        quitTitle = [[NSString stringWithCPPWString:Strings::getString(StringId::quit)] uppercaseString];
        extrasTitle = [[NSString stringWithCPPWString:Strings::getString(StringId::extras)] uppercaseString];
        tutorialTitle = [[NSString stringWithCPPWString:Strings::getString(StringId::tutorial)] uppercaseString];
    }
    
    [self.btnNewGame setTitle:newGameTitle forState:UIControlStateNormal];
    [self.btnContinue setTitle:continueTitle forState:UIControlStateNormal];
    [self.btnLoad setTitle:loadTitle forState:UIControlStateNormal];
    [self.btnSave setTitle:saveTitle forState:UIControlStateNormal];
    [self.btnSettings setTitle:settingsTitle forState:UIControlStateNormal];
    [self.btnQuit setTitle:quitTitle forState:UIControlStateNormal];
    [self.btnTutorial setTitle:tutorialTitle forState:UIControlStateNormal];
    [self.btnExtras setTitle:extrasTitle forState:UIControlStateNormal];
    
    [self.btnNewGame configureWithTheme];
    [self.btnContinue configureWithTheme];
    [self.btnLoad configureWithTheme];
    [self.btnSave configureWithTheme];
    [self.btnSettings configureWithTheme];
    [self.btnQuit configureWithTheme];
    [self.btnExtras configureWithTheme];
    [self.btnTutorial configureWithTheme];
}

-(void)viewWillAppear:(BOOL)animated
{
    [super viewWillAppear:animated];
    
    [self configureControls];
}

-(void)configureControls
{
    self.btnContinue.enabled = self.nativeUIInterface->isContinueEnabled()?YES:NO;
    self.btnSave.enabled = self.nativeUIInterface->isSaveEnabled()?YES:NO;
    self.btnLoad.enabled = self.nativeUIInterface->isLoadEnabled()?YES:NO;
}

- (IBAction)newGameTapped:(UIButton *)sender {
    [buttonClickPlayer() play];

    self.nativeUIInterface->onNewGamePressed();
}
- (IBAction)saveTapped:(id)sender {
    [buttonClickPlayer() play];

    self.nativeUIInterface->onSavePressed();
}
- (IBAction)loadTapped:(id)sender {
    [buttonClickPlayer() play];

    self.nativeUIInterface->onLoadPressed();
}
- (IBAction)continueTapped:(id)sender {
    [buttonClickPlayer() play];

    self.nativeUIInterface->onContinuePressed();
}
- (IBAction)settingsTapped:(id)sender {
    [buttonClickPlayer() play];

    self.nativeUIInterface->onSettingsFromMainMenu();
    
    [self.navigationController pushViewController:[SettingsViewController settingsViewController:self.nativeUIInterface] animated:NO];
}
- (IBAction)tutorialTapped:(id)sender
{
    [buttonClickPlayer() play];

    // show tutorial video
    self.nativeUIInterface->showTutorialVideo();
    
}
- (IBAction)extrasTapped:(id)sender {
    
    vector<ExtraItem> extraItems = self.nativeUIInterface->getExtras();
    
    ExtraItemsListViewController* extrasList = [ExtraItemsListViewController extrasListWithItems:extraItems];
    
    [self.navigationController pushViewController:extrasList animated:NO];
}

- (IBAction)quitTapped:(id)sender
{
    [buttonClickPlayer() play];

    self.nativeUIInterface->onQuitPressed();
}

-(IBAction)share:(id)sender
{
    UIActivityViewController* activityViewController = [UIActivityViewController shareMyAppActivityViewController];
    
    BOOL isIpad = [UIDevice currentDevice].userInterfaceIdiom == UIUserInterfaceIdiomPad;

    
    if (isIpad) {
        UIPopoverController* popoverController = [[UIPopoverController alloc] initWithContentViewController:activityViewController];
        
        [popoverController presentPopoverFromRect:[sender frame] inView:self.view permittedArrowDirections:UIPopoverArrowDirectionAny animated:YES];
    }
    else
    {
        [self presentViewController:activityViewController animated:YES completion:nil];
    }
}



@end
