    /* ScummVM - Graphic Adventure Engine
 *
 * ScummVM is the legal property of its developers, whose names
 * are too numerous to list here. Please refer to the COPYRIGHT
 * file distributed with this source distribution.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 */

// Disable symbol overrides so that we can use system headers.
#define FORBIDDEN_SYMBOL_ALLOW_ALL

#define OBJC_CODE


#include "iphone_video.h"

#include "graphics/colormasks.h"

#include "osys_main.h"

#include "enhanced/PlatformInterface.h"
#include "enhanced/constants/Constants.h"

iPhoneView *g_iPhoneViewInstance = nil;
static int g_fullWidth;
static int g_fullHeight;

static int g_needsScreenUpdate = 0;

#if 0
static long g_lastTick = 0;
static int g_frames = 0;
#endif

#define printOpenGLError() printOglError(__FILE__, __LINE__)

Common::String getGLErrStr(GLenum error) {
    switch (error) {
        case GL_INVALID_ENUM:
            return "GL_INVALID_ENUM";
        case GL_INVALID_VALUE:
            return "GL_INVALID_VALUE";
        case GL_INVALID_OPERATION:
            return "GL_INVALID_OPERATION";
        case GL_OUT_OF_MEMORY:
            return "GL_OUT_OF_MEMORY";
    }

    return Common::String::format("(Unknown GL error code 0x%X)", error);
}

int printOglError(const char *file, int line) {
	int retCode = 0;

	// returns 1 if an OpenGL error occurred, 0 otherwise.
	GLenum glErr = glGetError();
	while (glErr != GL_NO_ERROR) {
		fprintf(stderr, "glError: %s: %u (%s: %d)\n", getGLErrStr(glErr).c_str(), glErr, file, line);
		retCode = 1;
		glErr = glGetError();
	}
	return retCode;
}


#ifndef M_PI
#define M_PI 3.141592653589793238f
#endif
#define DEG2RAD (M_PI / 180.0f)


static const GLfloat identity_matrix[16] = {
	1.0f, 0.0f, 0.0f, 0.0f,
	0.0f, 1.0f, 0.0f, 0.0f,
	0.0f, 0.0f, 1.0f, 0.0f,
	0.0f, 0.0f, 0.0f, 1.0f
};

#define A(row,col)  a[(col<<2)+row]
#define B(row,col)  b[(col<<2)+row]
#define P(row,col)  product[(col<<2)+row]


//from mesa3d:
static void matmul4( GLfloat *product, const GLfloat *a, const GLfloat *b )
{
	GLint i;
	for (i = 0; i < 4; i++) {
		const GLfloat ai0=A(i,0),  ai1=A(i,1),  ai2=A(i,2),  ai3=A(i,3);
		P(i,0) = ai0 * B(0,0) + ai1 * B(1,0) + ai2 * B(2,0) + ai3 * B(3,0);
		P(i,1) = ai0 * B(0,1) + ai1 * B(1,1) + ai2 * B(2,1) + ai3 * B(3,1);
		P(i,2) = ai0 * B(0,2) + ai1 * B(1,2) + ai2 * B(2,2) + ai3 * B(3,2);
		P(i,3) = ai0 * B(0,3) + ai1 * B(1,3) + ai2 * B(2,3) + ai3 * B(3,3);
	}
}

#undef A
#undef B
#undef P

//from mesa3d:
void matrix_translate(GLfloat *m, GLfloat x, GLfloat y, GLfloat z)
{
	m[12] = m[0] * x + m[4] * y + m[8]  * z + m[12];
	m[13] = m[1] * x + m[5] * y + m[9]  * z + m[13];
	m[14] = m[2] * x + m[6] * y + m[10] * z + m[14];
	m[15] = m[3] * x + m[7] * y + m[11] * z + m[15];
}


void setOrtho(float m[16], float left, float right, float bottom, float top)
{
	const float near = 0.0f;
	const float far = 1.0f;
	float m2[16];

#define M(row,col)  m2[col*4+row]
	M(0,0) = 2.0f / (right-left);
	M(0,1) = 0.0f;
	M(0,2) = 0.0f;
	M(0,3) = -(right+left) / (right-left);

	M(1,0) = 0.0f;
	M(1,1) = 2.0f / (top-bottom);
	M(1,2) = 0.0f;
	M(1,3) = -(top+bottom) / (top-bottom);

	M(2,0) = 0.0f;
	M(2,1) = 0.0f;
	M(2,2) = -2.0f / (far-near);
	M(2,3) = -(far+near) / (far-near);

	M(3,0) = 0.0f;
	M(3,1) = 0.0f;
	M(3,2) = 0.0f;
	M(3,3) = 1.0f;
#undef M

	matmul4(m, m, m2);
}


bool iPhone_isHighResDevice() {
	return g_fullHeight > 480;
}

void iPhone_updateScreen() {
    
    static UIApplicationState last_state = UIApplicationStateActive;
    UIApplicationState cur_state=[[UIApplication sharedApplication] applicationState];
    cur_state = cur_state == UIApplicationStateActive ? UIApplicationStateActive : UIApplicationStateInactive;
    if (last_state != cur_state)
    {
        if (cur_state==UIApplicationStateActive)
        {
            NSLog(@"OMER: Resumed");
            ((OSystem_IPHONE*)g_system)->startSoundsystem();
        }
        else
        {
            NSLog(@"OMER: Going to suspend");
             ((OSystem_IPHONE*)g_system)->stopSoundsystem();
        }
        
        
        last_state = cur_state;
    }
    
    if (UIApplicationStateActive != cur_state) return; //don't draw when running from BG
    
	//printf("Mouse: (%i, %i)\n", mouseX, mouseY);
	if (!g_needsScreenUpdate) {
		g_needsScreenUpdate = 1;
		[g_iPhoneViewInstance performSelectorOnMainThread:@selector(updateSurface) withObject:nil waitUntilDone: YES];
	}
}

void iPhone_updateOverlayOnly() {
    
    static UIApplicationState last_state = UIApplicationStateActive;
    UIApplicationState cur_state=[[UIApplication sharedApplication] applicationState];
    cur_state = cur_state == UIApplicationStateActive ? UIApplicationStateActive : UIApplicationStateInactive;
    
    if (UIApplicationStateActive != cur_state) return; //don't draw when running from BG
    
    if (!g_needsScreenUpdate) {
        g_needsScreenUpdate = 1;
        [g_iPhoneViewInstance performSelectorOnMainThread:@selector(updateEnhancedOverlay) withObject:nil waitUntilDone: YES];
    }

}


bool iPhone_fetchEvent(InternalEvent *event) {
	return [g_iPhoneViewInstance fetchEvent:event];
}

uint getSizeNextPOT(uint size) {
	if ((size & (size - 1)) || !size) {
		int log = 0;

		while (size >>= 1)
			++log;

		size = (2 << log);
	}

	return size;
}

const char *iPhone_getDocumentsDir() {
	NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
	NSString *documentsDirectory = [paths objectAtIndex:0];
	return [documentsDirectory UTF8String];
}

@implementation iPhoneView

+ (Class)layerClass {
	return [CAEAGLLayer class];
}

- (VideoContext *)getVideoContext {
	return &_videoContext;
}

- (void)resetContext {
    [EAGLContext setCurrentContext:_context];
}

- (void)createContext {
	CAEAGLLayer *eaglLayer = (CAEAGLLayer *)self.layer;

	eaglLayer.opaque = YES;
	eaglLayer.drawableProperties = [NSDictionary dictionaryWithObjectsAndKeys:
	                                [NSNumber numberWithBool:FALSE], kEAGLDrawablePropertyRetainedBacking, kEAGLColorFormatRGB565, kEAGLDrawablePropertyColorFormat, nil];

	_context = [[EAGLContext alloc] initWithAPI:kEAGLRenderingAPIOpenGLES2];

	// In case creating the OpenGL ES context failed, we will error out here.
	if (_context == nil) {
		fprintf(stderr, "Could not create OpenGL ES context\n");
		exit(-1);
	}

	if ([EAGLContext setCurrentContext:_context]) {
        
        
        
		glGenFramebuffers(1, &_viewFramebuffer); printOpenGLError();
		glGenRenderbuffers(1, &_viewRenderbuffer); printOpenGLError();

		glBindFramebuffer(GL_FRAMEBUFFER, _viewFramebuffer); printOpenGLError();
		glBindRenderbuffer(GL_RENDERBUFFER, _viewRenderbuffer); printOpenGLError();
		[_context renderbufferStorage:GL_RENDERBUFFER fromDrawable:(id<EAGLDrawable>)self.layer];

		glFramebufferRenderbuffer(GL_FRAMEBUFFER, GL_COLOR_ATTACHMENT0, GL_RENDERBUFFER, _viewRenderbuffer); printOpenGLError();

		// Retrieve the render buffer size. This *should* match the frame size,
		// i.e. g_fullWidth and g_fullHeight.
		glGetRenderbufferParameteriv(GL_RENDERBUFFER, GL_RENDERBUFFER_WIDTH, &_renderBufferWidth); printOpenGLError();
		glGetRenderbufferParameteriv(GL_RENDERBUFFER, GL_RENDERBUFFER_HEIGHT, &_renderBufferHeight); printOpenGLError();

		if (glCheckFramebufferStatus(GL_FRAMEBUFFER) != GL_FRAMEBUFFER_COMPLETE) {
			NSLog(@"Failed to make complete framebuffer object %x.", glCheckFramebufferStatus(GL_FRAMEBUFFER));
			return;
		}

		_videoContext.overlayHeight = _renderBufferWidth;
		_videoContext.overlayWidth = _renderBufferHeight;
		uint overlayTextureWidth = getSizeNextPOT(_videoContext.overlayWidth);
		uint overlayTextureHeight = getSizeNextPOT(_videoContext.overlayHeight);

		// Since the overlay size won't change the whole run, we can
		// precalculate the texture coordinates for the overlay texture here
		// and just use it later on.
		_overlayTexCoords[2] = _overlayTexCoords[6] = _videoContext.overlayWidth / (GLfloat)overlayTextureWidth;
		_overlayTexCoords[5] = _overlayTexCoords[7] = _videoContext.overlayHeight / (GLfloat)overlayTextureHeight;

		_videoContext.overlayTexture.create(overlayTextureWidth, overlayTextureHeight, Graphics::createPixelFormat<8888>());

		glViewport(0, 0, _renderBufferWidth, _renderBufferHeight); printOpenGLError();
		glClearColor(0.0f, 0.0f, 0.0f, 1.0f); printOpenGLError();

		glEnable(GL_BLEND); printOpenGLError();
		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA); printOpenGLError();

        //
        // Generate framebuffer and texture for scaling
        //
        glGenFramebuffers(1, &_scalingFramebuffer); printOpenGLError();
        glGenTextures(1, &_scalingTexture); printOpenGLError();
        glBindTexture(GL_TEXTURE_2D, _scalingTexture); printOpenGLError();
        
        // Decide on the dimensions for scaling texture
        int scalingFactor = MAX(_renderBufferWidth / GAME_SCREEN_WIDTH, _renderBufferHeight / GAME_SCREEN_HEIGHT);
        _scaledContentWidth = GAME_SCREEN_WIDTH * scalingFactor;
        _scaledContentHeight = GAME_SCREEN_HEIGHT * scalingFactor;
        _scalingTextureWidth = getSizeNextPOT(_scaledContentWidth);
        _scalingTextureHeight = getSizeNextPOT(_scaledContentHeight);
        
        // Init texture parameters
        glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, _scalingTextureWidth, _scalingTextureHeight, 0, GL_RGB, GL_UNSIGNED_SHORT_5_6_5, 0); printOpenGLError();
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE); printOpenGLError();
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE); printOpenGLError();
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST); printOpenGLError();
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST); printOpenGLError();
 
        // Attach texture to framebuffer
        glBindFramebuffer(GL_FRAMEBUFFER, _scalingFramebuffer); printOpenGLError();
        glFramebufferTexture2D(GL_FRAMEBUFFER, GL_COLOR_ATTACHMENT0, GL_TEXTURE_2D, _scalingTexture, 0); printOpenGLError();
        
        // check for framebuffer completion
        GLuint status = glCheckFramebufferStatus(GL_FRAMEBUFFER); printOpenGLError();
        if (status != GL_FRAMEBUFFER_COMPLETE) {
            LOGE("Unable to create framebuffer");
        }
        
        glBindFramebuffer(GL_FRAMEBUFFER, _viewFramebuffer); printOpenGLError();
        glBindRenderbuffer(GL_RENDERBUFFER, _viewRenderbuffer); printOpenGLError();
	}

    fprintf(stderr, "Context Has been created\n");
}

- (id)initWithFrame:(struct CGRect)frame {
	self = [super initWithFrame: frame];

	_contentScaleFactor = 1;
	if ([[UIScreen mainScreen] respondsToSelector:@selector(scale)]) {
		if ([self respondsToSelector:@selector(setContentScaleFactor:)]) {
			// Horrible and crazy method to get the proper return value of
			// scale when the SDK used for building does not know anything
			// about the selector scale...
			NSMethodSignature *scaleSignature = [UIScreen instanceMethodSignatureForSelector:@selector(scale)];
			NSInvocation *scaleInvocation = [NSInvocation invocationWithMethodSignature:scaleSignature];
			[scaleInvocation setTarget:[UIScreen mainScreen]];
			[scaleInvocation setSelector:@selector(scale)];
			[scaleInvocation invoke];

			NSInteger returnLength = [[scaleInvocation methodSignature] methodReturnLength];
			if (returnLength == sizeof(CGFloat)) {
				[scaleInvocation getReturnValue:&_contentScaleFactor];
				[self setContentScaleFactor:_contentScaleFactor];
			}
		}
	}

	g_fullWidth = (int)frame.size.width;
	g_fullHeight = (int)frame.size.height;

	g_iPhoneViewInstance = self;

	_keyboardView = nil;
	_screenTexture = 0;
	_overlayTexture = 0;
	_mouseCursorTexture = 0;

	_scaledShakeOffsetY = 0;

	_firstTouch = NULL;
	_secondTouch = NULL;

	_eventLock = [[NSLock alloc] init];

	_gameScreenVertCoords[0] = _gameScreenVertCoords[1] =
	    _gameScreenVertCoords[2] = _gameScreenVertCoords[3] =
	    _gameScreenVertCoords[4] = _gameScreenVertCoords[5] =
	    _gameScreenVertCoords[6] = _gameScreenVertCoords[7] = 0;

	_gameScreenTexCoords[0] = _gameScreenTexCoords[1] =
	    _gameScreenTexCoords[2] = _gameScreenTexCoords[3] =
	    _gameScreenTexCoords[4] = _gameScreenTexCoords[5] =
	    _gameScreenTexCoords[6] = _gameScreenTexCoords[7] = 0;

	_overlayVertCoords[0] = _overlayVertCoords[1] =
	    _overlayVertCoords[2] = _overlayVertCoords[3] =
	    _overlayVertCoords[4] = _overlayVertCoords[5] =
	    _overlayVertCoords[6] = _overlayVertCoords[7] = 0;

	_overlayTexCoords[0] = _overlayTexCoords[1] =
	    _overlayTexCoords[2] = _overlayTexCoords[3] =
	    _overlayTexCoords[4] = _overlayTexCoords[5] =
	    _overlayTexCoords[6] = _overlayTexCoords[7] = 0;

	_mouseVertCoords[0] = _mouseVertCoords[1] =
	    _mouseVertCoords[2] = _mouseVertCoords[3] =
	    _mouseVertCoords[4] = _mouseVertCoords[5] =
	    _mouseVertCoords[6] = _mouseVertCoords[7] = 0;

	_mouseTexCoords[0] = _mouseTexCoords[1] =
	    _mouseTexCoords[2] = _mouseTexCoords[3] =
	    _mouseTexCoords[4] = _mouseTexCoords[5] =
	    _mouseTexCoords[6] = _mouseTexCoords[7] = 0;

	// Initialize the OpenGL ES context
	[self createContext];

	return self;
}

- (void)dealloc {
	if (_keyboardView != nil) {
		[_keyboardView dealloc];
	}

	_videoContext.screenTexture.free();
	_videoContext.overlayTexture.free();
	_videoContext.mouseTexture.free();

	[_eventLock dealloc];
	[super dealloc];
}

- (void)drawRect:(CGRect)frame {
#if 0
	if (g_lastTick == 0) {
		g_lastTick = time(0);
	}

	g_frames++;
	if (time(0) > g_lastTick) {
		g_lastTick = time(0);
		printf("FPS: %i\n", g_frames);
		g_frames = 0;
	}
#endif
}

- (void)setFilterModeForTexture:(GLuint)tex {
	if (!tex)
		return;

	glBindTexture(GL_TEXTURE_2D, tex); printOpenGLError();

	GLint filter = GL_NEAREST;
//yinon
	switch (_videoContext.graphicsMode) {
	case kGraphicsModeLinear:
		filter = GL_LINEAR;
		break;

	case kGraphicsModeNone:
		filter = GL_NEAREST;
		break;
	}

	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, filter); printOpenGLError();
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, filter); printOpenGLError();
	// We use GL_CLAMP_TO_EDGE here to avoid artifacts when linear filtering
	// is used. If we would not use this for example the cursor in Loom would
	// have a line/border artifact on the right side of the covered rect.
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE); printOpenGLError();
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE); printOpenGLError();
}

- (void)setGraphicsMode {
	[self setFilterModeForTexture:_screenTexture];
	[self setFilterModeForTexture:_overlayTexture];
	[self setFilterModeForTexture:_mouseCursorTexture];
}

- (void)flushScalingFramebuffer {
    
    if (UIApplicationStateActive != [[UIApplication sharedApplication] applicationState]) return;

    // Bind the final view framebuffer and the scaled texture
    glBindFramebuffer(GL_FRAMEBUFFER, _viewFramebuffer); printOpenGLError();
    glBindTexture(GL_TEXTURE_2D, _scalingTexture); printOpenGLError();
    
    glViewport(0, 0, _renderBufferWidth, _renderBufferHeight); printOpenGLError();

    glClear(GL_COLOR_BUFFER_BIT); printOpenGLError();

    
    if (_videoContext.defaultShader)
    {
        const ShaderPass &p = _videoContext.defaultShader->passes[0];
        
        glUseProgram(p.program); printOpenGLError();
        glUniform1i(p.textureLoc, 0); printOpenGLError();
        
        float inputw = _scaledContentWidth;
        float inputh = _scaledContentHeight;
        float outputw = _videoContext.overlayWidth;
        float outputh = _videoContext.overlayHeight;
        float texw = _scalingTextureWidth;
        float texh = _scalingTextureHeight;
        
        glUniform2f(p.textureSizeLoc, (GLfloat)texw, (GLfloat)texh); printOpenGLError();
        
        glUniform2f(p.textureFractLoc, (GLfloat)1/texw, (GLfloat)1/texh); printOpenGLError();
        
        glUniform2f(p.inputSizeLoc, inputw, inputh); printOpenGLError();
        
        glUniform2f(p.outputSizeLoc, outputw, outputh); printOpenGLError();
        
        glUniform1f(p.alphaFactorLoc, 1.0); printOpenGLError();

        glUniformMatrix4fv(p.projMat, 1, GL_FALSE, _videoContext.projMat); printOpenGLError();
        
        GLfloat displayTexCoords[4 * 2];
        displayTexCoords[0] = displayTexCoords[1] = displayTexCoords[3] =  displayTexCoords[4] = 0;
        displayTexCoords[2] = displayTexCoords[6] = _scaledContentWidth / (GLfloat)_scalingTextureWidth;
        displayTexCoords[5] = displayTexCoords[7] = _scaledContentHeight / (GLfloat)_scalingTextureHeight;

        glVertexAttribPointer(p.positionAttributeLoc, 2, GL_FLOAT, false, 0, _gameScreenVertCoords); printOpenGLError();
        glVertexAttribPointer(p.texCoordAttributeLoc, 2, GL_FLOAT, false, 0, displayTexCoords); printOpenGLError();
    }
   
    glDrawArrays(GL_TRIANGLE_STRIP, 0, 4); printOpenGLError();
}

- (void)updateEnhancedOverlay
{
    if (!g_needsScreenUpdate) {
        return;
    }
    g_needsScreenUpdate = 0;
    
    
    [self flushScalingFramebuffer];
    
    Enhanced::PlatformInterface::instance()->afterDrawGameTextureToScreen();

    glBindRenderbuffer(GL_RENDERBUFFER, _viewRenderbuffer); printOpenGLError();
    [_context presentRenderbuffer:GL_RENDERBUFFER];
}

- (void)updateSurface {
    if (!g_needsScreenUpdate) {
		return;
	}
	g_needsScreenUpdate = 0;
    
    if (!GameInfo::isSimonGame())
        Enhanced::PlatformInterface::instance()->beforeDrawGameTextureToScreen(&_videoContext.screenTexture);

   
    [self updateMainSurface];
    [self flushScalingFramebuffer];
    
    Enhanced::PlatformInterface::instance()->afterDrawGameTextureToScreen();

	/*if (_videoContext.overlayVisible || 1)
		[self updateOverlaySurface];
    else
        warning("");*/

    
    // Removed, the enhanced overlay draws a cursor PNG
	//if (_videoContext.mouseIsVisible)
		//[self updateMouseSurface];
    
#ifdef TARGET_TONYT
    if (_videoContext.mouseIsVisible)
        [self updateMouseSurface];
#endif

	glBindRenderbuffer(GL_RENDERBUFFER, _viewRenderbuffer); printOpenGLError();
	[_context presentRenderbuffer:GL_RENDERBUFFER];
}

- (void)notifyMouseMove {
	const GLint mouseX = (GLint)(_videoContext.mouseX * _mouseScaleX) - _mouseHotspotX;
	const GLint mouseY = (GLint)(_videoContext.mouseY * _mouseScaleY) - _mouseHotspotY;

	_mouseVertCoords[0] = _mouseVertCoords[4] = mouseX;
	_mouseVertCoords[1] = _mouseVertCoords[3] = mouseY;
	_mouseVertCoords[2] = _mouseVertCoords[6] = mouseX + _mouseWidth;
	_mouseVertCoords[5] = _mouseVertCoords[7] = mouseY + _mouseHeight;
}

- (void)updateMouseCursorScaling {
	CGRect *rect;
	int maxWidth, maxHeight;

	if (!_videoContext.overlayVisible) {
		rect = &_gameScreenRect;
		maxWidth = _videoContext.screenWidth;
		maxHeight = _videoContext.screenHeight;
	} else {
		rect = &_overlayRect;
		maxWidth = _videoContext.overlayWidth;
		maxHeight = _videoContext.overlayHeight;
	}

	if (!maxWidth || !maxHeight) {
		printf("WARNING: updateMouseCursorScaling called when screen was not ready (%d)!\n", _videoContext.overlayVisible);
		return;
	}

	_mouseScaleX = CGRectGetWidth(*rect) / (GLfloat)maxWidth;
	_mouseScaleY = CGRectGetHeight(*rect) / (GLfloat)maxHeight;

	_mouseWidth = (GLint)(_videoContext.mouseWidth * _mouseScaleX);
	_mouseHeight = (GLint)(_videoContext.mouseHeight * _mouseScaleY);

	_mouseHotspotX = (GLint)(_videoContext.mouseHotspotX * _mouseScaleX);
	_mouseHotspotY = (GLint)(_videoContext.mouseHotspotY * _mouseScaleY);

	// We subtract the screen offset to the hotspot here to simplify the
	// screen offset handling in the mouse code. Note the subtraction here
	// makes sure that the offset actually gets added to the mouse position,
	// since the hotspot offset is substracted from the position.
	_mouseHotspotX -= (GLint)CGRectGetMinX(*rect);
	_mouseHotspotY -= (GLint)CGRectGetMinY(*rect);

	// FIXME: For now we also adapt the mouse position here. In reality we
	// would be better off to also adjust the event position when switching
	// from overlay to game screen or vica versa.
	[self notifyMouseMove];
}

- (void)updateMouseCursor {
	if (_mouseCursorTexture == 0) {
		glGenTextures(1, &_mouseCursorTexture); printOpenGLError();
		[self setFilterModeForTexture:_mouseCursorTexture];
	}

	[self updateMouseCursorScaling];

	_mouseTexCoords[2] = _mouseTexCoords[6] = _videoContext.mouseWidth / (GLfloat)_videoContext.mouseTexture.w;
	_mouseTexCoords[5] = _mouseTexCoords[7] = _videoContext.mouseHeight / (GLfloat)_videoContext.mouseTexture.h;

	glBindTexture(GL_TEXTURE_2D, _mouseCursorTexture); printOpenGLError();
	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, _videoContext.mouseTexture.w, _videoContext.mouseTexture.h, 0, GL_RGBA, GL_UNSIGNED_SHORT_5_5_5_1, _videoContext.mouseTexture.getPixels()); printOpenGLError();
}

- (void)updateMainSurface {
    
    if (UIApplicationStateActive != [[UIApplication sharedApplication] applicationState]) return;
    
	glBindTexture(GL_TEXTURE_2D, _screenTexture); printOpenGLError();
    
    // Decide whether to use modified game pixels from the enhanced layer, or the normal ones
    Graphics::Surface* usedGamePixels = NULL;
    if (!GameInfo::isSimonGame())
        
        usedGamePixels = Enhanced::PlatformInterface::instance()->getModifiedGamePixels();
    if (usedGamePixels == NULL)
        usedGamePixels = &_videoContext.screenTexture;
    
    // Bind the scaling framebuffer
    glBindFramebuffer(GL_FRAMEBUFFER, _scalingFramebuffer); printOpenGLError();
    
    glViewport(0, 0, _scaledContentWidth, _scaledContentHeight); printOpenGLError();

    glClear(GL_COLOR_BUFFER_BIT); printOpenGLError();

    if (!Enhanced::PlatformInterface::instance()->shouldDrawGameScreen())
        return;
    
     glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, _videoContext.screenTexture.w, _videoContext.screenTexture.h, 0, GL_RGB, GL_UNSIGNED_SHORT_5_6_5, usedGamePixels->getPixels()); printOpenGLError();
    
    if (_videoContext.currentShader)
    {
        const ShaderPass &p = _videoContext.currentShader->passes[0];
        
        glUseProgram(p.program); printOpenGLError();
        glUniform1i(p.textureLoc, 0); printOpenGLError();
        
        float inputw = _videoContext.screenWidth;
        float inputh = _videoContext.screenHeight;
        float outputw = _scaledContentWidth;
        float outputh = _scaledContentHeight;
        float texw = _videoContext.screenTexture.w;
        float texh = _videoContext.screenTexture.h;
        
        glUniform2f(p.textureSizeLoc, (GLfloat)texw, (GLfloat)texh); printOpenGLError();
        
        glUniform2f(p.textureFractLoc, (GLfloat)1/texw, (GLfloat)1/texh); printOpenGLError();
        
        glUniform2f(p.inputSizeLoc, inputw, inputh); printOpenGLError();
        
        glUniform2f(p.outputSizeLoc, outputw, outputh); printOpenGLError();
        
        glUniform1f(p.alphaFactorLoc, 1.0); printOpenGLError();

        glUniformMatrix4fv(p.projMat, 1, GL_FALSE, _videoContext.scalingMat); printOpenGLError();
        
        GLfloat scaledVertCoords[4 * 2];
        scaledVertCoords[0] = scaledVertCoords[1] = scaledVertCoords[3] =  scaledVertCoords[4] = 0;
        scaledVertCoords[2] = scaledVertCoords[6] = _scaledContentWidth;
        scaledVertCoords[5] = scaledVertCoords[7] = _scaledContentHeight;

        
        glVertexAttribPointer(p.positionAttributeLoc, 2, GL_FLOAT, false, 0, scaledVertCoords); printOpenGLError();
        glVertexAttribPointer(p.texCoordAttributeLoc, 2, GL_FLOAT, false, 0, _gameScreenTexCoords); printOpenGLError();
    }
    
    // Unfortunately we have to update the whole texture every frame, since glTexSubImage2D is actually slower in all cases
	// due to the iPhone internals having to convert the whole texture back from its internal format when used.
	// In the future we could use several tiled textures instead.
	glDrawArrays(GL_TRIANGLE_STRIP, 0, 4); printOpenGLError();
}

- (void)updateOverlaySurface {
    glBindTexture(GL_TEXTURE_2D, _overlayTexture); printOpenGLError();
    glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, _videoContext.overlayTexture.w, _videoContext.overlayTexture.h, 0, GL_RGBA, GL_UNSIGNED_BYTE, _videoContext.overlayTexture.getPixels()); printOpenGLError();

    const ShaderPass &p = _videoContext.defaultShader->passes[0];
    glUseProgram(p.program); printOpenGLError();
    glUniform1i(p.textureLoc, 0); printOpenGLError();
    glUniformMatrix4fv(p.projMat, 1, GL_FALSE, _videoContext.projMat); printOpenGLError();

    glVertexAttribPointer(p.positionAttributeLoc, 2, GL_FLOAT, false, 0, _overlayVertCoords); printOpenGLError();
    glVertexAttribPointer(p.texCoordAttributeLoc, 2, GL_FLOAT, false, 0, _overlayTexCoords); printOpenGLError();
    
	glDrawArrays(GL_TRIANGLE_STRIP, 0, 4); printOpenGLError();
}

- (void)updateMouseSurface {
    glBindTexture(GL_TEXTURE_2D, _mouseCursorTexture); printOpenGLError();

    const ShaderPass &p = _videoContext.defaultShader->passes[0];
    glUseProgram(p.program); printOpenGLError();
    glUniform1i(p.textureLoc, 0); printOpenGLError();
    glUniformMatrix4fv(p.projMat, 1, GL_FALSE, _videoContext.projMat); printOpenGLError();

    glVertexAttribPointer(p.positionAttributeLoc, 2, GL_FLOAT, false, 0, _mouseVertCoords); printOpenGLError();
    glVertexAttribPointer(p.texCoordAttributeLoc, 2, GL_FLOAT, false, 0, _mouseTexCoords); printOpenGLError();
    
 /*   LOGD("updateMouseSurface: vert coords %f %f %f %f %f %f %f %f", _mouseVertCoords[0], _mouseVertCoords[1], _mouseVertCoords[2], _mouseVertCoords[3], _mouseVertCoords[4], _mouseVertCoords[5], _mouseVertCoords[6], _mouseVertCoords[7]);*/


	glDrawArrays(GL_TRIANGLE_STRIP, 0, 4); printOpenGLError();
}

- (void)setUpOrientation:(UIDeviceOrientation)orientation width:(int *)width height:(int *)height {
	_orientation = orientation;

    //_orientation =  UIDeviceOrientationLandscapeLeft;

	//set identity
	memcpy(_videoContext.projMat, identity_matrix, sizeof(identity_matrix));

    float angle;

	// We always force the origin (0,0) to be in the upper left corner.
	switch (_orientation) {
	case UIDeviceOrientationLandscapeRight:
		//set rotation
		angle = 0.0f;
		break;
	case UIDeviceOrientationLandscapeLeft:
    default:
        angle = 0.0f;
	}

    //set projection
    setOrtho(_videoContext.projMat, 0, _renderBufferWidth, _renderBufferHeight, 0);

    //translate
    matrix_translate(_videoContext.projMat, 0.0f, 0.0f, 0.0f);

    //rotate
    GLfloat rotation_matrix[16];

    //set to identity
    memcpy(rotation_matrix, identity_matrix, sizeof(identity_matrix));

    //set rotation
    GLfloat s = (GLfloat)sin(angle * DEG2RAD);
    GLfloat c = (GLfloat)cos(angle * DEG2RAD);
    rotation_matrix[0] = c;
    rotation_matrix[1] = s;
    rotation_matrix[4] = -s;
    rotation_matrix[5] = c;

    //apply rotation
    matmul4(_videoContext.projMat, _videoContext.projMat, rotation_matrix);

    *width = _renderBufferWidth;
    *height = _renderBufferHeight;
    
    
    
    //
    // Setup matrix for scaling framebuffer (always assumes same orientation)
    //
    memcpy(_videoContext.scalingMat, identity_matrix, sizeof(identity_matrix));
    setOrtho(_videoContext.scalingMat, 0, _scaledContentWidth, 0, _scaledContentHeight);
    matrix_translate(_videoContext.scalingMat, 0.0f, 0.0f, 0.0f);
}

- (void)createScreenTexture {
	const uint screenTexWidth = getSizeNextPOT(_videoContext.screenWidth);
	const uint screenTexHeight = getSizeNextPOT(_videoContext.screenHeight);

	_gameScreenTexCoords[2] = _gameScreenTexCoords[6] = _videoContext.screenWidth / (GLfloat)screenTexWidth;
	_gameScreenTexCoords[5] = _gameScreenTexCoords[7] = _videoContext.screenHeight / (GLfloat)screenTexHeight;
    
	_videoContext.screenTexture.create(screenTexWidth, screenTexHeight, Graphics::createPixelFormat<565>());
}

- (void)initSurface {
	int screenWidth, screenHeight;

    UIDeviceOrientation o = [[UIDevice currentDevice] orientation];

    //if ([[UIDevice currentDevice] orientation] == UIDeviceOrientationPortrait)
    //o = UIDeviceOrientationLandscapeLeft;

	[self setUpOrientation:o width:&screenWidth height:&screenHeight];

    _videoContext.overlayWidth = screenWidth;
    _videoContext.overlayHeight = screenHeight;

    if (Enhanced::PlatformInterface::instance() != NULL)
    {
        Enhanced::PlatformInterface::instance()->onSurfaceChanged(screenWidth, screenHeight);
    }

	if (_screenTexture > 0) {
		glDeleteTextures(1, &_screenTexture); printOpenGLError();
	}

	glGenTextures(1, &_screenTexture); printOpenGLError();
	[self setFilterModeForTexture:_screenTexture];

	if (_overlayTexture > 0) {
		glDeleteTextures(1, &_overlayTexture); printOpenGLError();
	}

	glGenTextures(1, &_overlayTexture); printOpenGLError();
	[self setFilterModeForTexture:_overlayTexture];
    
#ifdef TARGET_TONYT
    if (_mouseCursorTexture > 0) {
        glDeleteTextures(1, &_mouseCursorTexture); printOpenGLError();
    }
    
    glGenTextures(1, &_mouseCursorTexture); printOpenGLError();
    [self setFilterModeForTexture:_mouseCursorTexture];
#endif
    
	glBindRenderbuffer(GL_RENDERBUFFER, _viewRenderbuffer); printOpenGLError();

	[self clearColorBuffer];

	if (_keyboardView != nil) {
		[_keyboardView removeFromSuperview];
		[[_keyboardView inputView] removeFromSuperview];
	}

	GLfloat adjustedWidth = _videoContext.screenHeight;
	GLfloat adjustedHeight = _videoContext.screenWidth;

	/*
    if (_videoContext.asprectRatioCorrection) {
		if (_videoContext.screenWidth == 320 && _videoContext.screenHeight == 200)
			adjustedHeight = 240;
		else if (_videoContext.screenWidth == 640 && _videoContext.screenHeight == 400)
			adjustedHeight = 480;
	}
     */

	float overlayPortraitRatio;
    //yinon
	if (1 || (_orientation == UIDeviceOrientationLandscapeLeft || _orientation ==  UIDeviceOrientationLandscapeRight) && 1) {
		GLfloat gameScreenRatio = adjustedWidth / adjustedHeight;
		GLfloat screenRatio = (GLfloat)screenWidth / (GLfloat)screenHeight;

        printf("Screen W=%d , H=%d, adjustedWidth = %f , adjustedHeight =%f\n",screenWidth,screenHeight,adjustedWidth,adjustedHeight);


		// These are the width/height according to the portrait layout!
		int rectWidth, rectHeight;
		int xOffset, yOffset;

		if (gameScreenRatio < screenRatio && 0) {
			// When the game screen ratio is less than the screen ratio
			// we need to scale the width, since the game screen was higher
			// compared to the width than our output screen is.
			rectWidth = (int)(screenHeight * gameScreenRatio);
            rectWidth = screenWidth;
            rectHeight = screenHeight;
			xOffset = (screenWidth - rectWidth) / 2;
			yOffset = 0;
		} else {
			// When the game screen ratio is bigger than the screen ratio
			// we need to scale the height, since the game screen was wider
			// compared to the height than our output screen is.
			rectWidth = screenWidth;
			rectHeight = screenHeight;

            xOffset =  0;

			yOffset = (screenHeight - rectHeight) / 2;
		}

		//printf("Rect: %i, %i, %i, %i\n", xOffset, yOffset, rectWidth, rectHeight);
		_gameScreenRect = CGRectMake(xOffset, yOffset, rectWidth, rectHeight);
		overlayPortraitRatio = 1.0f;
	} else {
		GLfloat ratio = adjustedHeight / adjustedWidth;
		int height = (int)(screenWidth * ratio);
		//printf("Making rect (%u, %u)\n", screenWidth, height);
		_gameScreenRect = CGRectMake(0, 0, screenWidth, height);

		CGRect keyFrame = CGRectMake(0.0f, 0.0f, 0.0f, 0.0f);
		if (_keyboardView == nil) {
			_keyboardView = [[SoftKeyboard alloc] initWithFrame:keyFrame];
			[_keyboardView setInputDelegate:self];
		}

		[self addSubview:[_keyboardView inputView]];
		[self addSubview: _keyboardView];
		[[_keyboardView inputView] becomeFirstResponder];

		overlayPortraitRatio = (_videoContext.overlayHeight * ratio) / _videoContext.overlayWidth;
	}

	_overlayRect = CGRectMake(0, 0, screenWidth, screenHeight * overlayPortraitRatio);

	_gameScreenVertCoords[0] = _gameScreenVertCoords[4] = CGRectGetMinX(_gameScreenRect);
	_gameScreenVertCoords[1] = _gameScreenVertCoords[3] = CGRectGetMinY(_gameScreenRect);
	_gameScreenVertCoords[2] = _gameScreenVertCoords[6] = CGRectGetMaxX(_gameScreenRect);
	_gameScreenVertCoords[5] = _gameScreenVertCoords[7] = CGRectGetMaxY(_gameScreenRect);

	_overlayVertCoords[2] = _overlayVertCoords[6] = CGRectGetMaxX(_overlayRect);
	_overlayVertCoords[5] = _overlayVertCoords[7] = CGRectGetMaxY(_overlayRect);

	[self setViewTransformation];
	[self updateMouseCursorScaling];
}

- (void)setViewTransformation {
#if 0
	// Set the modelview matrix. This matrix will be used for the shake offset
	// support.
	gldLoadIdentity(_videoContext.projMat);

	// Scale the shake offset according to the overlay size. We need this to
	// adjust the overlay mouse click coordinates when an offset is set.
	_scaledShakeOffsetY = (int)(_videoContext.shakeOffsetY / (GLfloat)_videoContext.screenHeight * CGRectGetHeight(_overlayRect));

	// Apply the shakeing to the output screen.
	gldTranslatef(_videoContext.projMat, 0, -_scaledShakeOffsetY, 0);
#endif
}

- (void)clearColorBuffer {
	// The color buffer is triple-buffered, so we clear it multiple times right away to avid doing any glClears later.
	int clearCount = 5;
	while (clearCount-- > 0) {
		glClear(GL_COLOR_BUFFER_BIT); printOpenGLError();
		[_context presentRenderbuffer:GL_RENDERBUFFER];
	}
}

- (void)addEvent:(InternalEvent)event {
	[_eventLock lock];
	_events.push_back(event);
	[_eventLock unlock];
}

- (bool)fetchEvent:(InternalEvent *)event {
	[_eventLock lock];
	if (_events.empty()) {
		[_eventLock unlock];
		return false;
	}

	*event = *_events.begin();
	_events.pop_front();
	[_eventLock unlock];
	return true;
}

/**
 * Converts portrait mode coordinates into rotated mode coordinates.
 */
- (bool)convertToRotatedCoords:(CGPoint)point result:(CGPoint *)result {
	switch (_orientation) {
	case UIDeviceOrientationLandscapeLeft:
		result->x = point.y;
		result->y = _renderBufferWidth - point.x;
		return true;

	case UIDeviceOrientationLandscapeRight:
		result->x = _renderBufferHeight - point.y;
		result->y = point.x;
		return true;

	case UIDeviceOrientationPortrait:
		result->x = point.x;
		result->y = point.y;
		return true;

	default:
		return false;
	}
}

- (bool)getMouseCoords:(CGPoint)point eventX:(int *)x eventY:(int *)y {
	// We scale the input according to our scale factor to get actual screen
	// cooridnates.
	point.x *= _contentScaleFactor;
	point.y *= _contentScaleFactor;

	//if (![self convertToRotatedCoords:point result:&point])
	//	return false;

	CGRect *area;
	int width, height, offsetY;
	if (_videoContext.overlayVisible) {
		area = &_overlayRect;
		width = _videoContext.overlayWidth;
		height = _videoContext.overlayHeight;
		offsetY = _scaledShakeOffsetY;
	} else {
		area = &_gameScreenRect;
		width = _videoContext.screenWidth;
		height = _videoContext.screenHeight;
		offsetY = _videoContext.shakeOffsetY;
	}

	point.x = (point.x - CGRectGetMinX(*area)) / CGRectGetWidth(*area);
	point.y = (point.y - CGRectGetMinY(*area)) / CGRectGetHeight(*area);

	*x = (int)(point.x * width);
	// offsetY describes the translation of the screen in the upward direction,
	// thus we need to add it here.
	*y = (int)(point.y * height + offsetY);

	// Clip coordinates
	if (*x < 0 || *x > width || *y < 0 || *y > height)
		return false;

	return true;
}

- (void)deviceOrientationChanged:(UIDeviceOrientation)orientation {
    switch (orientation) {
	case UIDeviceOrientationLandscapeLeft:
	case UIDeviceOrientationLandscapeRight:
	case UIDeviceOrientationPortrait:
		_orientation = orientation;
		break;

	default:
		return;
	}

//    printf("orientation=%d\n",orientation);

	[self addEvent:InternalEvent(kInputOrientationChanged, orientation, 0)];
}

- (void)touchesBegan:(NSSet *)touches withEvent:(UIEvent *)event {
	NSSet *allTouches = [event allTouches];
	int x, y;

    self.lastTouchBeginTime=[NSDate date];
    
	switch ([allTouches count]) {
	case 1: {
		UITouch *touch = [touches anyObject];
		CGPoint point = [touch locationInView:self];
        
        // First, pass the event to the enhanced layer
        // OMER: is this the right way? (multiply by _contentScaleFactor and apply rotation)
        CGPoint converted = point;
        converted.x *= _contentScaleFactor;
        converted.y *= _contentScaleFactor;
      //  [self convertToRotatedCoords:converted result:&converted];
        
        if (Enhanced::PlatformInterface::instance()->onDownEvent(converted.x, converted.y))
        {
            _firstTouch = touch;
            return;
        }
        
		if (![self getMouseCoords:point eventX:&x eventY:&y])
			return;

		_firstTouch = touch;
		[self addEvent:InternalEvent(kInputMouseDown, x, y)];
		break;
		}

	case 2: {
		UITouch *touch = [touches anyObject];
		CGPoint point = [touch locationInView:self];
		if (![self getMouseCoords:point eventX:&x eventY:&y])
			return;

		_secondTouch = touch;
		[self addEvent:InternalEvent(kInputMouseSecondDown, x, y)];
		break;
		}
	}
}

- (void)touchesMoved:(NSSet *)touches withEvent:(UIEvent *)event {
	//NSSet *allTouches = [event allTouches];
	int x, y;
    
    for (UITouch *touch in touches) {
		if (touch == _firstTouch) {
			CGPoint point = [touch locationInView:self];
            if ([touch locationInView:self].x == [touch previousLocationInView:self].x
                && [touch locationInView:self].y == [touch previousLocationInView:self].y)
            {
                return; //its a force touch event ignore this press
            }
            // First, pass the event to the enhanced layer
            // OMER: is this the right way? (multiply by _contentScaleFactor and apply rotation)
            CGPoint converted = point;
            converted.x *= _contentScaleFactor;
            converted.y *= _contentScaleFactor;
         //   [self convertToRotatedCoords:converted result:&converted];
            
            // First, pass the event to the enhanced layer
            if (Enhanced::PlatformInterface::instance()->onScrollEvent(converted.x, converted.y))
                return;
            
			if (![self getMouseCoords:point eventX:&x eventY:&y])
				return;

			[self addEvent:InternalEvent(kInputMouseDragged, x, y)];
		} else if (touch == _secondTouch) {
			CGPoint point = [touch locationInView:self];
			if (![self getMouseCoords:point eventX:&x eventY:&y])
				return;

			[self addEvent:InternalEvent(kInputMouseSecondDragged, x, y)];
		}
	}
}

- (void)touchesEnded:(NSSet *)touches withEvent:(UIEvent *)event {
	NSSet *allTouches = [event allTouches];
	int x, y;

	switch ([allTouches count]) {
	case 1: {
		UITouch *touch = [[allTouches allObjects] objectAtIndex:0];
		CGPoint point = [touch locationInView:self];
        
        // First, pass the event to the enhanced layer
        // OMER: is this the right way? (multiply by _contentScaleFactor and apply rotation)
        CGPoint converted = point;
        converted.x *= _contentScaleFactor;
        converted.y *= _contentScaleFactor;
       // [self convertToRotatedCoords:converted result:&converted];
        
        // First, pass the event to the enhanced layer
        if (Enhanced::PlatformInterface::instance()->onUpEvent(converted.x, converted.y))
            return;
        
		if (![self getMouseCoords:point eventX:&x eventY:&y])
			return;

		[self addEvent:InternalEvent(kInputMouseUp, x, y)];
		break;
		}

	case 2: {
		UITouch *touch = [[allTouches allObjects] objectAtIndex:1];
		CGPoint point = [touch locationInView:self];
		if (![self getMouseCoords:point eventX:&x eventY:&y])
			return;

		[self addEvent:InternalEvent(kInputMouseSecondUp, x, y)];
		break;
		}
	}
}

- (void)touchesCancelled:(NSSet *)touches withEvent:(UIEvent *)event {
    NSSet *allTouches = [event allTouches];
    if ([allTouches count] == 1) {
        UITouch *touch = [[allTouches allObjects] objectAtIndex:0];
        CGPoint point = [touch locationInView:self];
        
        // First, pass the event to the enhanced layer
        // OMER: is this the right way? (multiply by _contentScaleFactor and apply rotation)
        CGPoint converted = point;
        converted.x *= _contentScaleFactor;
        converted.y *= _contentScaleFactor;
    //    [self convertToRotatedCoords:converted result:&converted];

        
        // Treat as an "up" event in the enhanced layer
        Enhanced::PlatformInterface::instance()->onUpEvent(converted.x, converted.y);
    }
}

- (void)tapGesture:(CGPoint)location {
    
    // OMER: is this the right way? (multiply by _contentScaleFactor and apply rotation)
    CGPoint converted = location;
    converted.x *= _contentScaleFactor;
    converted.y *= _contentScaleFactor;
   // [self convertToRotatedCoords:converted result:&converted];
    
    Enhanced::PlatformInterface::instance()->onTapEvent(converted.x, converted.y);
}


- (void)handleKeyPress:(unichar)c {
	[self addEvent:InternalEvent(kInputKeyPressed, c, 0)];
}

- (BOOL)canHandleSwipes {
	return TRUE;
}

- (int)swipe:(int)num withEvent:(struct __GSEvent *)event {
	//printf("swipe: %i\n", num);

	[self addEvent:InternalEvent(kInputSwipe, num, 0)];
	return 0;
}

- (void)applicationSuspend {
	[self addEvent:InternalEvent(kInputApplicationSuspended, 0, 0)];
}

- (void)applicationResume {
	[self addEvent:InternalEvent(kInputApplicationResumed, 0, 0)];
}

@end
