/*
 * Drawable.h
 *
 *  Created on: Jun 23, 2013
 *      Author: omergilad
 */

#ifndef DRAWABLE_H_
#define DRAWABLE_H_

#include <memory>
#include <enhanced/OverlayBitmap.h>

#include "Animation.h"

// MARK: temp: t7g
typedef void ShaderProgram;

class Animation;
typedef std::shared_ptr<Animation> AnimationPtr;


class Drawable {

public:

	Drawable();

	virtual ~Drawable();

	float getAlpha() const;

	void setAlpha(float alpha);

	AnimationPtr getAnimation() const;

	void setAnimation(AnimationPtr animation);

    Enhanced::OverlayBitmap* getBitmap() const;

	void setBitmap(Enhanced::OverlayBitmap* bitmap);

	ShaderProgram* getProgram() const;
	void setProgram(ShaderProgram* program);

	float getPositionX() const;

	void setPositionX(float positionX);

	float getPositionY() const;

	void setPositionY(float positionY);

	virtual float getWidth() const;

	void setWidth(float width);

	virtual float getHeight() const;

	void setHeight(float width);

	void updateAnimation(long currentTime);

	bool isAnimationFinished();

	virtual void takeBitmapSnapshot();

	static void setDisplayRatio(float ratio);

	inline bool shouldCenter() const {
		return mShouldCenter;
	}

	void setShouldCenter(bool shouldCenter) {
		mShouldCenter = shouldCenter;
	}

protected:

	bool mUsingSnapshot;

private:

	Enhanced::OverlayBitmap* mBitmap;
	float mPositionX;
	float mPositionY;
	float mWidth;
	float mHeight;
	float mAlpha;
	bool mShouldCenter;
	AnimationPtr mAnimation;

	ShaderProgram* mShaderProgram;

	static float sDisplayRatio;

};

typedef std::shared_ptr<Drawable> DrawablePtr;

#endif /* DRAWABLE_H_ */
